﻿using ProjectDumpTruck.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectDumpTruck.Drawnings;

public static class ExtentionDrawningTruck
{
    /// <summary>
	/// Разделитель для записи информации по объекту в файл
	/// </summary>
	private static readonly string _separatorForObject = ":";

    /// <summary>
    /// Создание объекта из строки
    /// </summary>
    /// <param name="info">Строка с данными для создания объекта</param>
    /// <returns>Объект</returns>
    public static DrawningTruck? CreateDrawningTruck(this string info)
    {
        string[] strs = info.Split(_separatorForObject);
        EntityTruck? truck = EntityDumpTruck.CreateEntityDumpTruck(strs);
        if (truck != null)
        {
            return new DrawningDumpTruck((EntityDumpTruck)truck);
        }

        truck = EntityTruck.CreateEntityTruck(strs);
        if (truck != null)
        {
            return new DrawningTruck(truck);
        }

        return null;
    }

    /// <summary>
    /// Получение данных для сохранения в файл
    /// </summary>
    /// <param name="drawningCar">Сохраняемый объект</param>
    /// <returns>Строка с данными по объекту</returns>
    public static string GetDataForSave(this DrawningTruck drawningTruck)
    {
        string[]? array = drawningTruck?.EntityTruck?.GetStringRepresentation();

        if (array == null)
        {
            return string.Empty;
        }

        return string.Join(_separatorForObject, array);
    }
}
