﻿namespace ProjectDumpTruck.Entities;

/// <summary>
/// Класс-сущность "Самосвал"
/// </summary>
public class EntityDumpTruck : EntityTruck
{
    /// <summary>
    /// Дополнительный цвет (для опциональных элементов)
    /// </summary>
    public Color AdditionalColor { get; private set; }
    public void SetAdditionalColor(Color color)
    {
        AdditionalColor = color;
    }

    /// <summary>
    /// Дополнительный цвет (для опциональных элементов)
    /// </summary>
    public Color Additional2Color { get; private set; }
    public void SetAdditional2Color(Color color)
    {
        Additional2Color = color;
    }

    /// <summary>
    /// Признак (опция) наличия кузова
    /// </summary>
    public bool Bodywork { get; private set; }

    /// <summary>
    /// Признак (опция) наличия тента
    /// </summary>
    public bool Awning { get; private set; }

    /// <summary>
    /// Инициализация полей объекта-класса самосвала
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="additional2Color">Дополнительный цвет</param>
    /// <param name="bodywork">Признак наличия кузова</param>
    /// <param name="awning">Признак наличия тента</param>

    public EntityDumpTruck(int speed, double weight, Color bodyColor, Color additionalColor, Color additional2Color, bool bodywork, bool awning) : base(5, 45, bodyColor)
    {
        AdditionalColor = additionalColor;
        Additional2Color = additional2Color;
        Bodywork = bodywork;
        Awning = awning;
    }

    /// <summary>
    /// Получение строк со значениями свойств продвинутого объекта класса-сущности
    /// </summary>
    /// <returns></returns>
    public override string[] GetStringRepresentation()
    {
        return new[] { nameof(EntityDumpTruck), Speed.ToString(), Weight.ToString(), BodyColor.Name, AdditionalColor.Name,
                        Additional2Color.Name, Bodywork.ToString(), Awning.ToString()};
    }
    /// <summary>
    /// Создание продвинутого объекта из массива строк
    /// </summary>
    /// <param name="strs"></param>
    /// <returns></returns>
    public static EntityDumpTruck? CreateEntityDumpTruck(string[] strs)
    {
        if (strs.Length != 8 || strs[0] != nameof(EntityDumpTruck))
        {
            return null;
        }
        return new EntityDumpTruck(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]),
                                    Color.FromName(strs[4]), Color.FromName(strs[5]), Convert.ToBoolean(strs[6]), Convert.ToBoolean(strs[7]));
    }
}